@@ -3,14 +3,14 @@
 import ooolib
 
 # Create the document
-doc = ooolib.Calc()
+doc = ooolib.Calc("First")
 
 # By default you start on Sheet1.  This has an index of 0 in ooolib.
 doc.set_cell_value(1, 1, "string", "We start on \"Sheet1\"")
 
 # Create a new sheet by passing the title.  You will automatically
 # move to that sheet.
-doc.new_sheet("Sheet2")
+doc.new_sheet("Second")
 doc.set_cell_value(1, 1, "string", "I'm on Sheet2")
 
 # Create another one
