@@ -42,6 +42,17 @@ doc.set_cell_property('background', 'default')
 doc.set_cell_value(2, 3, "string", "Default Colors")
 
 
+# Text Font Sizes
+doc.set_cell_property('fontsize', '10')
+doc.set_cell_value(3, 1, "string", "Default 10pt")
+
+doc.set_cell_property('fontsize', '11')
+doc.set_cell_value(3, 2, "string", "11pt")
+
+doc.set_cell_property('fontsize', '12')
+doc.set_cell_value(3, 3, "string", "12pt")
+
+doc.set_cell_property('fontsize', '10')
 
 # Write out the document
 doc.save("calc-example05.ods")
