@@ -1,7 +1,7 @@
-"ooolib-python - Copyright (C) 2006-2007 Joseph Colton"
+"ooolib-python - Copyright (C) 2006-2008 Joseph Colton"
 
 # ooolib-python - Python module for creating Open Document Format documents.
-# Copyright (C) 2006-2007  Joseph Colton
+# Copyright (C) 2006-2008  Joseph Colton
 
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -20,16 +20,17 @@
 # You can contact me by email at josephcolton@gmail.com
 
 # Import Standard Modules
-import zipfile
+import zipfile           # Needed for reading/writing documents
 import time
 import sys
 import glob
 import os
 import re
+import xml.parsers.expat # Needed for parsing documents
 
 def version():
 	"Get the ooolib-python version"
-	return "ooolib-python-0.0.11"
+	return "ooolib-python-0.0.13"
 
 def clean_string(data):
 	"Returns an XML friendly copy of the data string"
@@ -163,9 +164,14 @@ class XML:
 class Meta:
 	"Meta Data Class"
 
-	def __init__(self, doctype):
+	def __init__(self, doctype, debug=False):
 		self.doctype = doctype
-		self.meta_generator = version()       # The generator should always default to the version number
+
+		# Set the debug mode
+		self.debug = debug
+
+		# The generator should always default to the version number
+		self.meta_generator = version()
 		self.meta_title = ''
 		self.meta_subject = ''
 		self.meta_description = ''
@@ -182,6 +188,11 @@ class Meta:
 		self.meta_user4_value = ''
 		self.meta_creation_date = self.meta_time()
 
+		# Parser data
+		self.parser_element_list = []
+		self.parser_element = ""
+		self.parser_count = 0
+
 	def set_meta(self, metaname, value):
 		"""Set meta data in your document.
 
@@ -205,12 +216,98 @@ class Meta:
 			if value not in self.meta_keywords:
 				self.meta_keywords.append(value)
 
+	def get_meta_value(self, metaname):
+		"Get meta data value for a given metaname."
+
+		if metaname == 'creator': return self.meta_creator
+		if metaname == 'editor': return self.meta_editor
+		if metaname == 'title': return self.meta_title
+		if metaname == 'subject': return self.meta_subject
+		if metaname == 'description': return self.meta_description
+		if metaname == 'user1name': return self.meta_user1_name
+		if metaname == 'user2name': return self.meta_user2_name
+		if metaname == 'user3name': return self.meta_user3_name
+		if metaname == 'user4name': return self.meta_user4_name
+		if metaname == 'user1value': return self.meta_user1_value
+		if metaname == 'user2value': return self.meta_user2_value
+		if metaname == 'user3value': return self.meta_user3_value
+		if metaname == 'user4value': return self.meta_user4_value
+		if metaname == 'keyword': return self.meta_keywords
+
 	def meta_time(self):
 		"Return time string in meta data format"
 		t = time.localtime()
 		stamp = "%04d-%02d-%02dT%02d:%02d:%02d" % (t[0], t[1], t[2], t[3], t[4], t[5])
 		return stamp
 
+	def parse_start_element(self, name, attrs):
+		if self.debug: print '* Start element:', name
+		self.parser_element_list.append(name)
+		self.parser_element = self.parser_element_list[-1]
+
+		# Need the meta name from the user-defined tags
+		if (self.parser_element == "meta:user-defined"):
+			self.parser_count += 1
+			# Set user-defined name
+			self.set_meta("user%dname" % self.parser_count, attrs['meta:name'])
+
+		# Debugging statements
+		if self.debug: print "  List: ", self.parser_element_list
+		if self.debug: print "  Attributes: ", attrs
+
+
+	def parse_end_element(self, name):
+		if self.debug: print '* End element:', name
+		if name != self.parser_element:
+			print "Tag Mismatch: '%s' != '%s'" % (name, self.parser_element)
+		self.parser_element_list.pop()
+
+		# Readjust parser_element_list and parser_element
+		if (self.parser_element_list):
+			self.parser_element = self.parser_element_list[-1]
+		else:
+			self.parser_element = ""
+
+	def parse_char_data(self, data):
+		if self.debug: print "  Character data: ", repr(data)
+
+		# Collect Meta data fields
+		if (self.parser_element == "dc:title"):
+			self.set_meta("title", data)
+		if (self.parser_element == "dc:description"):
+			self.set_meta("description", data)
+		if (self.parser_element == "dc:subject"):
+			self.set_meta("subject", data)
+		if (self.parser_element == "meta:initial-creator"):
+			self.set_meta("creator", data)
+
+		# Try to maintain the same creation date
+		if (self.parser_element == "meta:creation-date"):
+			self.meta_creation_date = data
+		
+		# The user defined fields need to be kept track of, parser_count does that
+		if (self.parser_element == "meta:user-defined"):
+			self.set_meta("user%dvalue" % self.parser_count, data)
+
+	def meta_parse(self, data):
+		"Parse Meta Data from a meta.xml file"
+
+		# Debugging statements
+		if self.debug:
+			# Sometimes it helps to see the document that was read from
+			print data
+			print "\n\n\n"
+
+		# Create parser
+		parser = xml.parsers.expat.ParserCreate()
+		# Set up parser callback functions
+		parser.StartElementHandler = self.parse_start_element
+		parser.EndElementHandler = self.parse_end_element
+		parser.CharacterDataHandler = self.parse_char_data
+
+		# Actually parse the data
+		parser.Parse(data, 1)
+
 	def get_meta(self):
 		"Generate meta.xml file data"
 		self.meta_date = self.meta_time()
@@ -510,6 +607,10 @@ class CalcSheet:
 		self.max_col = 0
 		self.max_row = 0
 
+	def get_sheet_dimensions(self):
+		"Returns the max column and row"
+		return (self.max_col, self.max_row)
+
 	def clean_formula(self, data):
 		"Returns a formula for use in ODF"
 		# Example Translations
@@ -533,10 +634,22 @@ class CalcSheet:
 		"Returns the sheet name"
 		return self.sheet_name
 
+	def set_name(self, sheetname):
+		"Resets the sheet name"
+		self.sheet_name = sheetname
+
 	def get_sheet_values(self):
 		"Returns the sheet cell values"
 		return self.sheet_values
 
+	def get_sheet_value(self, col, row):
+		"Get the value contents of a cell"
+		cell = (col, row)
+		if cell in self.sheet_values:
+			return self.sheet_values[cell]
+		else:
+			return None
+
 	def get_sheet_config(self):
 		"Returns the sheet cell properties"
 		return self.sheet_config
@@ -701,10 +814,10 @@ class CalcSheet:
 
 class Calc:
 	"Calc Class - Used to create OpenDocument Format Calc Spreadsheets."
-	def __init__(self, sheetname=None):
+	def __init__(self, sheetname=None, opendoc=None, debug=False):
 		"Initialize ooolib Calc instance"
 		# Default to no debugging
-		self.debug = False
+		self.debug = debug
 		if not sheetname: sheetname = "Sheet1"
 		self.sheets = [CalcSheet(sheetname)]  # The main sheet will be initially called 'Sheet1'
 		self.sheet_index = 0                  # We initially start on the first sheet
@@ -715,6 +828,23 @@ class Calc:
 		self.styles.get_style_code('table')   # Force generation of default table
 		self.styles.get_style_code('cell')    # Force generation of default cell
 
+		# Data Parsing
+		self.parser_element_list = []
+		self.parser_element = ""
+		self.parser_sheet_num = 0
+		self.parser_sheet_row = 0
+		self.parser_sheet_column = 0
+		self.parser_cell_string_pending = False
+		self.parser_cell_string_line = ""
+
+		# See if we need to read a document
+		if opendoc:
+			# Verify that the document exists
+			if self.debug: print "Opening Document: %s" % opendoc
+
+			# Okay, now we load the file
+			self.load(opendoc)
+
 	def debug_level(self, level):
 		"""Set debug level:
 		True if you want debugging messages
@@ -726,6 +856,18 @@ class Calc:
 		"Set meta data in your document."
 		self.meta.set_meta(metaname, value)
 
+	def get_meta_value(self, metaname):
+		"Get meta data value for a given metaname"
+		return self.meta.get_meta_value(metaname)
+
+	def get_sheet_name(self):
+		"Returns the sheet name"
+		return self.sheets[self.sheet_index].get_name()
+
+	def get_sheet_dimensions(self):
+		"Returns the sheet dimensions in (cols, rows)"
+		return self.sheets[self.sheet_index].get_sheet_dimensions()
+
 	def set_column_property(self, column, name, value):
 		"Set Column Properties"
 		if name == 'width':
@@ -759,6 +901,10 @@ class Calc:
 				self.sheet_index = index
 		return self.sheet_index
 
+	def get_sheet_count(self):
+		"Returns the number of existing sheets"
+		return len(self.sheets)
+
 	def new_sheet(self, sheetname):
 		"Create a new sheet"
 		self.sheet_index = len(self.sheets)
@@ -771,6 +917,138 @@ class Calc:
 		style_code = self.styles.get_style_code('cell')
 		self.sheets[self.sheet_index].set_sheet_config((col, row), style_code)
 
+	def get_cell_value(self, col, row):
+		"Get a cell value tuple (type, value) for a given cell"
+		sheetvalue = self.sheets[self.sheet_index].get_sheet_value(col, row)
+		# We stop here if there is no value for sheetvalue
+		if sheetvalue == None: return sheetvalue
+		# Now check to see if we have a value tuple
+		if 'value' in sheetvalue:
+			return sheetvalue['value']
+		else:
+			return None
+
+	def load(self, filename):
+		"""Load .ods spreadsheet.
+
+		The load function loads data from a document into the current cells.
+		"""
+		# Read in the important files
+
+		# meta.xml
+		data = self._zip_read(filename, "meta.xml")
+		self.meta.meta_parse(data)		
+
+		# content.xml
+		data = self._zip_read(filename, "content.xml")
+		self.content_parse(data)
+
+		# settings.xml - I do not remember putting anything here
+		# styles.xml - I do not remember putting anything here
+
+	def parse_content_start_element(self, name, attrs):
+		if self.debug: print '* Start element:', name
+		self.parser_element_list.append(name)
+		self.parser_element = self.parser_element_list[-1]
+
+		# Keep track of the current sheet number
+		if (self.parser_element == 'table:table'):
+			# Move to starting cell
+			self.parser_sheet_row = 0
+			self.parser_sheet_column = 0
+			# Increment the sheet number count
+			self.parser_sheet_num += 1
+			if (self.parser_sheet_num - 1 != self.sheet_index):
+				# We are not on the first sheet and need to create a new sheet.
+				# We will automatically move to the new sheet
+				sheetname = "Sheet%d" % self.parser_sheet_num
+				if 'table:name' in attrs: sheetname = attrs['table:name']
+				self.new_sheet(sheetname)
+			else:
+				# We are on the first sheet and will need to overwrite the default name
+				sheetname = "Sheet%d" % self.parser_sheet_num
+				if 'table:name' in attrs: sheetname = attrs['table:name']
+				self.sheets[self.sheet_index].set_name(sheetname)
+
+		# Update the row numbers
+		if (self.parser_element == 'table:table-row'):
+			self.parser_sheet_row += 1
+			self.parser_sheet_column = 0
+
+		# Okay, now keep track of the sheet cell data
+		if (self.parser_element == 'table:table-cell'):
+			# We must be in a new column
+			self.parser_sheet_column += 1
+			# Set some default values
+			datatype = ""
+			value = ""
+			# Get values from attrs hash
+			if 'office:value-type' in attrs: datatype = attrs['office:value-type']
+			if 'office:value' in attrs: value = attrs['office:value']
+			if 'table:formula' in attrs:
+				datatype = 'formula'
+				value = attrs['table:formula']
+			if datatype == 'string':
+				datatype = ""
+				self.parser_cell_string_pending = True
+				self.parser_cell_string_line = ""
+			# Set the cell value
+			if datatype:
+				self.set_cell_value(self.parser_sheet_column, self.parser_sheet_row, datatype, value)
+
+		# Debugging statements
+		if self.debug: print "  List: ", self.parser_element_list
+		if self.debug: print "  Attributes: ", attrs
+
+
+	def parse_content_end_element(self, name):
+		if self.debug: print '* End element:', name
+		if name != self.parser_element:
+			print "Tag Mismatch: '%s' != '%s'" % (name, self.parser_element)
+		self.parser_element_list.pop()
+
+		# If the element was text:p and we are in string mode
+		if (self.parser_element == 'text:p'):
+			if (self.parser_cell_string_pending):
+				self.parser_cell_string_pending = False
+
+
+		# Readjust parser_element_list and parser_element
+		if (self.parser_element_list):
+			self.parser_element = self.parser_element_list[-1]
+		else:
+			self.parser_element = ""
+
+	def parse_content_char_data(self, data):
+		if self.debug: print "  Character data: ", repr(data)
+
+		if (self.parser_element == 'text:p'):
+			if (self.parser_cell_string_pending):
+				# Set the string and leave string pending mode
+				# This does feel a little kludgy, but it does the job
+				self.parser_cell_string_line = "%s%s" % (self.parser_cell_string_line, data)
+				self.set_cell_value(self.parser_sheet_column, self.parser_sheet_row,
+						    'string', self.parser_cell_string_line)
+
+	def content_parse(self, data):
+		"Parse Content Data from a content.xml file"
+
+		# Debugging statements
+		if self.debug:
+			# Sometimes it helps to see the document that was read from
+			print data
+			print "\n\n\n"
+
+		# Create parser
+		parser = xml.parsers.expat.ParserCreate()
+		# Set up parser callback functions
+		parser.StartElementHandler = self.parse_content_start_element
+		parser.EndElementHandler = self.parse_content_end_element
+		parser.CharacterDataHandler = self.parse_content_char_data
+
+		# Actually parse the data
+		parser.Parse(data, 1)
+
 	def save(self, filename):
 		"""Save .ods spreadsheet.
 
@@ -794,12 +1072,20 @@ class Calc:
 		self._zip_insert(self.savefile, "styles.xml", self._ods_styles())
 
 	def _zip_insert(self, file, filename, data):
+		"Insert a file into the zip archive"
+
 		now = time.localtime(time.time())[:6]
 		info = zipfile.ZipInfo(filename)
 		info.date_time = now
 		info.compress_type = zipfile.ZIP_DEFLATED
 		file.writestr(info, data)
 
+	def _zip_read(self, file, filename):
+		"Get the data from a file in the zip archive by filename"
+		file = zipfile.ZipFile(file, "r")
+		data = file.read(filename)
+		return data
+
 	def _ods_content(self):
 		"Generate ods content.xml data"
 
